/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;
using cmpMassPing;

namespace MassTools.NetworkAwareness.AvailableBW
{
    public class AvailableBW_ICMPing : MassTool
    {
        public event MassToolDataReadyEventHandler DataReadyEvent;

       	private Int32 m_nPacketSize = 64;
        private Int32 m_nPacketNum = 3;
        private Int32 m_nInterval = 500;

        CMassPing cping = null;

        public AvailableBW_ICMPing()
        {
            cping = new CMassPing();
            cping.PingReadyEvent += new CMassPing.PingReadyEventHandler(cping_PingReadyEvent);
            cping.Timeout = 3000;
            cping.Frequency = 1000;
        }

        public override bool Start()
        {
            cping.Start();
            return true;
        }

        public override bool Stop()
        {
            cping.Stop();
            return true;
        }

        public void SetPacketSize(Int32 n)
        {
            m_nPacketSize = n;
        }

        public void SetPacketNum(Int32 n)
        {
            m_nPacketNum = n;
            cping.Timeout = m_nPacketNum * m_nInterval;
        }

        public void SetInterval(Int32 n)
        {
            m_nInterval = n;
            cping.Frequency = m_nInterval;
            cping.Timeout = m_nPacketNum * m_nInterval;
        }

        public void SetRemoteAddr(string strAddr)
        {
            cping.RemoteAddress = strAddr;
        }

        public void AddSubscriber(MassToolDataReadyEventHandler handler)
        {
            DataReadyEvent += handler;
        }

        public void cping_PingReadyEvent(object sender, PingEventArgs e)
        {
            if (!e.m_bResult)
            {
                DataReadyEvent(this, new MassToolEventArgs(8));
                Console.WriteLine("\n" + e.m_strErr + "\n");
            }
            else if (e.m_nTime > m_nPacketNum * m_nInterval)
            {
                DataReadyEvent(this, new MassToolEventArgs(8));
                Console.WriteLine("\ntime: " + e.m_nTime + "; ttl: " + e.m_nTTL + "\n");
            }
            else
            {
                DataReadyEvent(this, new MassToolEventArgs(12));
                Console.WriteLine("\ntime: " + e.m_nTime + "; ttl: " + e.m_nTTL + "\n");
            }
        }

    }
}
