/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Runtime.InteropServices;

namespace MassTools.NetworkAwareness.AvailableBW
{
    class PTRServer
    {
        [DllImport("kernel32.dll")]
        extern static short QueryPerformanceCounter(ref long x);
        [DllImport("kernel32.dll")]
        extern static short QueryPerformanceFrequency(ref long x);


        const double PhaseWait = 0.3; // wait for next packets for 1s
        const int NoDataWait = 600;	// wait time when there is no data coming
        const double GetPktTimeOut = 0.3; // wait time when there is no data coming
        const int MaxClientNum = 64; // this is used for filter cmd set up
        const int MaxNoDataNum = 3;	// number of phases allowed to be no data
        
        string version = "2.1";
        
        class filter_item {
            public string src_ip_str; //char src_ip_str[16];
            public string dst_ip_str; //char dst_ip_str[16];
            
            public IPAddress src_ip; //uint32 src_ip;
            public IPAddress dst_ip; //uint32 dst_ip;
            
            public Socket control_sock;
            
            public int listen_port;
            public Socket listen_sock;
            
            public Common.pkt_rcd_t[] record = new Common.pkt_rcd_t[Common.MaxProbeNum];
            
            // last active time            
            public long pre_time;

            // the number of packets we have filtered out for this filter
            public int count;
            
            // the number of packets client should send to us
            public int probe_num;
            
            // number of consecutive no data phase, 3 means dead
            public int nodata_count;
            
            // working state
            public bool dead;

            public Thread thread = null;
            
            public filter_item next;
        };
        
        filter_item filter_list = null;
        static Mutex filter_mutex = new Mutex(); //pthread_mutex_t filter_mutex;
        
        Socket sock;
        IPEndPoint dst_addr, src_addr;
        
        IPAddress src_ip, dst_ip;
        int src_port, dst_port;

        string src_ip_str, dst_ip_str; // char src_ip_str[16], dst_ip_str[16];
        int src_size, dst_size;
        
        int probe_num = 60;
        bool verbose = false;
        bool debug = false;

        long tick_freq = 0;

        bool bend = false;

        public PTRServer()
        {
            QueryPerformanceFrequency(ref tick_freq);
        }

        public PTRServer(bool d, bool v, bool h)
        {
            QueryPerformanceFrequency(ref tick_freq);

            debug = d;
            verbose = v;
        }

        public bool Start()
        {
            // Create dst socket
            bend = false;
            try
            {
                sock = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            }
            catch
            {
                Console.WriteLine("Unable to open socket for listener (2):");
                Environment.Exit(1);
            }
            
            dst_port = Common.START_PORT + 1;            

            while (dst_port < Common.END_PORT)
            {
                try
                {
                    dst_addr = new IPEndPoint(IPAddress.Any, dst_port);
                    sock.Bind((EndPoint)dst_addr);
                    break;
                }
                catch
                {
                    dst_port ++;
                }
            }
            if (verbose) {
                Console.WriteLine("server port = {0}", dst_port);
            }
            
            sock.Listen(5);
            
            // starts listening command from clients
            while (!bend) {
                check_client();
            }
            
            sock.Close();

            return true;
        }

        public bool Stop()
        {
            bend = true;
            sock.Close();
            cleanup(0);
            return true;
        }

        /* usage message */
        void Usage()
        {
            Console.WriteLine("IGI/PTR-{0} server usage:\n", version);
            Console.WriteLine("\tigi-server [-vd]\n");
            Console.WriteLine("\t-h\tprint this message.");
            Console.WriteLine("\t-v\tverbose mode.");
            Console.WriteLine("\t-d\tdebug mode (dump more message than the verbose mode).\n");
            Environment.Exit(1);
        }
        
        /* make a clean exit on interrupts */
        void cleanup(int signo)
        {
            filter_item pfilter;
	        filter_item pfilter_post;

            pfilter = filter_list;
            
            while (pfilter != null) 
            {
                pfilter_post = pfilter.next;
                
                if (!(pfilter.dead))
                    pfilter.control_sock.Close();
                pfilter = null;
                
                pfilter = pfilter_post;
            }

            GC.Collect();
            //Environment.Exit(0);
        }

        /* get the current time */
        long get_time()
        {
            long count = 0;
            QueryPerformanceCounter(ref count); // Begin timing.
            return count;
        }

        double get_time_span(long s_timetick, long e_timetick)
        {
            return (double)(e_timetick - s_timetick) / tick_freq;
        }

        /* check if we have got all the packets for client's one-phase probing 
         * and can send back the filter packet information */
        void phase_finish(filter_item fi)
        {
            // Phase end condition: 1. get probe_num probing packets; 2. no new packet after PhaseWait time            
            long cur_time = get_time();
            filter_item ptr;

            byte[] msg_buf = new byte[64];
            int msg_len;
            int data_size; 

            filter_mutex.WaitOne(); //pthread_mutex_lock(&filter_mutex);

            if (fi == null)
            {
                ptr = filter_list;
            }
            else
            {
                ptr = fi;
            }

            for (; ptr != null; ptr = ptr.next) 
            {
                if (ptr.dead) continue;
                
                if (ptr.count == 0) 
                { 
                    if (get_time_span(ptr.pre_time, cur_time) > NoDataWait) 
                    {
                        // this is a no data phase
                        ptr.nodata_count ++;
                        
                        if (ptr.nodata_count >= MaxNoDataNum) 
                        {
                            ptr.control_sock.Close();
                            if (verbose) 
                            {
                                Console.WriteLine("{0} no data phases, close socket with {1}", MaxNoDataNum, ptr.src_ip_str);
                            }
                            ptr.dead = true;
                            
                            filter_mutex.ReleaseMutex();
                            Thread.CurrentThread.Abort(); // return;
                        }
                        else {
                            if (verbose) {
                                Console.WriteLine("start sending back 0 data\n");
                            }
                            
                            data_size = Common.pkt_rcd_t.Size() * ptr.count;		
                            // send the the control information to tell how many record will be transmitted back
                            string tempstr = "$" + data_size + "$";
                            msg_buf = Encoding.ASCII.GetBytes(tempstr);

                            msg_len = msg_buf.Length;

                            ptr.control_sock.Send(msg_buf, msg_len, SocketFlags.None);
                            
                            if (debug) {
                                Console.WriteLine("send out msg: | {0} |", tempstr);
                            }
                            
                            // send back the record information
                            byte[] recbuff = new byte[data_size];
                            for (int n = 0; n < ptr.count; n++)
                            {
                                Array.Copy(ptr.record[n].ToByteArray(), 0, recbuff, n * Common.pkt_rcd_t.Size(), Common.pkt_rcd_t.Size());
                            }
                            ptr.control_sock.Send(recbuff, data_size, SocketFlags.None);
                            ptr.pre_time = get_time();
                        }
                    }
                    continue;
                }
                
                // we have got some trace packets
                if ((ptr.count == ptr.probe_num) || (get_time_span(ptr.pre_time, cur_time) > PhaseWait)) 
                {
                    // clear previous bad record
                    ptr.nodata_count = 0;
                    
                    if (verbose) {
                        Console.WriteLine("start sending back data");
                    }
                    
                    data_size = Common.pkt_rcd_t.Size() * ptr.count;
                    
                    // send the the control information to tell how many record will be transmitted back
                    string tempstr = "$" + data_size + "$";
                    msg_buf = Encoding.ASCII.GetBytes(tempstr);
                    msg_len = msg_buf.Length;

                    ptr.control_sock.Send(msg_buf, msg_len, SocketFlags.None);
                    
                    if (debug) {
                        Console.WriteLine("send out msg: | {0} |", tempstr);
                    }
                    
                    // send back the record information
                    byte[] recbuff = new byte[data_size];
                    for (int n = 0; n < ptr.count; n++)
                    {
                        Array.Copy(ptr.record[n].ToByteArray(), 0, recbuff, n * Common.pkt_rcd_t.Size(), Common.pkt_rcd_t.Size());
                    }
                    ptr.control_sock.Send(recbuff, data_size, SocketFlags.None);
                    
                    // ready for the next phase
                    ptr.count = 0;
                    ptr.pre_time = get_time();
                }
            }
            filter_mutex.ReleaseMutex(); // pthread_mutex_unlock(&filter_mutex);
        }
        
        /* the main function of child process, filter probing packets from clients */
        void get_packets(filter_item ptr)
        {
            IPEndPoint client_addr = new IPEndPoint(IPAddress.Any, 0);
            EndPoint ep_client = (EndPoint)client_addr;
            
            byte[] recv_buf = new byte[4096]; // char recv_buf[4096];
            int cur_len;	
            long cur_time;
            
            if (verbose) {
                Console.WriteLine("come into get_packets ");
            }
            
            for (;;) {
                if (true == ptr.listen_sock.Poll((int)(1000000 * GetPktTimeOut), SelectMode.SelectRead)) 
                {
                    cur_len = ptr.listen_sock.ReceiveFrom(recv_buf, ref ep_client);
                    
                    ptr.record[ptr.count].seq = recv_buf[0];
                    cur_time = get_time();
                    ptr.record[ptr.count].tick = cur_time;
                    
                    if (debug)
                        Console.WriteLine("{0} {1} {2}", ptr.count, ptr.record[ptr.count].tick, tick_freq);
                    
                    ptr.count ++;
                    ptr.pre_time = cur_time;
                }
                phase_finish(ptr);
            }
        }

        /* If the <src_ip, dst_ip> pair is already in the pdev list, then just 
         * update its active_time, and return the point; otherwise, create a new 
         * filter item, and initilize it return the udp listening port number */
        void update_filter_list(Socket newsd, int probe_num, string strSrcIP, string strDstIP, out int listening_port)
        {
            filter_item p, pre;
            long cur_time;
            
            filter_mutex.WaitOne();
            
            // check whether any of the filter item has been idle long enough to be considered "finish", 
            // or have no data to receive
            p = filter_list;
            pre = null;
            
            cur_time = get_time();
            while (p != null) {
                if (p.dead) {
                    // delete this item 
                    if (verbose) {
                        Console.WriteLine("delete filter item {0} {1} ",p.src_ip_str, p.dst_ip_str);
                    }
                    
                    if (pre == null)
                        filter_list = filter_list.next;
                    else
                        pre.next = p.next;
                    
                    p = null;
                    GC.Collect();
                    
                    if (pre == null)
                        p = filter_list;
                    else
                        p = pre.next;
                }
                else {
                    pre = p;
                    p = p.next;
                }
            }
            
            // check if src & dst ip pair already in the filter list
            p = filter_list;
            pre = null;
            while (p != null) {
                if (p.src_ip_str.CompareTo(strSrcIP) == 0 && p.dst_ip_str.CompareTo(strDstIP) == 0)
                    break;
                else {
                    pre = p;
                    p = p.next;
                }
            }
            
            if (p != null) {
                if (verbose) {
                    Console.WriteLine("filter item already exits");
                }
                p.control_sock.Close();
                p.control_sock = newsd;
                p.pre_time = get_time();
                p.count = 0;
                p.probe_num = probe_num;
                p.nodata_count = 0;
                p.dead = false;
            }
            else {
                // for new ip pair, append in the filter_list
                IPEndPoint my_addr;
                
                if (verbose) {
                    Console.WriteLine("create a new filter item");
                }
                
                p = new filter_item();
                p.src_ip_str = strSrcIP;
                p.dst_ip_str = strDstIP;
                p.src_ip = IPAddress.Parse(strSrcIP);
                p.dst_ip = IPAddress.Parse(strDstIP);
                p.control_sock = newsd;
                p.pre_time = get_time();
                p.count = 0;
                p.probe_num = probe_num;
                p.nodata_count = 0;
                p.dead = false;
                
                // get the listening sock and port for the
                try {
                    p.listen_sock = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
                }
                catch {
                    Console.WriteLine("Error: server: unable to open socket.");
                    Environment.Exit(1);
                }
                
                dst_port ++;

                while (dst_port < Common.END_PORT)
                {
                    try
                    {
                        my_addr = new IPEndPoint(IPAddress.Any, dst_port);
                        p.listen_sock.Bind(my_addr); 
                        break;
                    }
                    catch (SocketException e)
                    {
                        Console.WriteLine("error: " + e.ToString());
                        dst_port ++;
                    }
                }
                
                p.listen_port = dst_port;
                
                p.next = null;
                
                if (pre == null) 
                {
                    // this is the first filter, don't need to worry about pc init
                    filter_list = p;
                }
                else 
                {
                    pre.next = p;
                }
                //ThreadStart starter = delegate { Fetch (myUrl); };
                //new Thread(starter).Start();
                ThreadStart starter = delegate { get_packets (p); };
                p.thread = new Thread(starter);
                p.thread.Start();
            }
            
            listening_port = p.listen_port;
            
            filter_mutex.ReleaseMutex();	    //pthread_mutex_unlock(&filter_mutex);
        }

        /* check if there is any new client making connection */
        void check_client()
        {	    
            byte[] msg_buf = new byte[1024];
            int msg_len;
            Socket newsd;
            int listening_port;
            
            try 
            {
                newsd = sock.Accept(); // accept(sock, (struct sockaddr *) &src_addr, &src_size);
            }
            catch
            {
                return;
            }

            // get src addresses
            src_ip = ((IPEndPoint)(newsd.RemoteEndPoint)).Address;
            src_ip_str = src_ip.ToString();
            
            // get the dst ip address that client is connecting with
            dst_ip = ((IPEndPoint)(newsd.LocalEndPoint)).Address;
            dst_ip_str = dst_ip.ToString();
            
            if (verbose) {
                Console.WriteLine("get new connection ");
                Console.WriteLine("src_ip_str = {0} ", src_ip_str);
                Console.WriteLine("dst_ip_str = {0} ", dst_ip_str);
                Console.WriteLine("waiting for START msg");
            }
            
            // waiting for "$START$%d$", for purpose of sanity check.
            // Here, the %d is the probe_num set by probing client
            
            msg_len = 0;
            while ((msg_len <= 7) || (msg_buf[msg_len-1] != '$')) {
                int tmp_len; 
                
                tmp_len = newsd.Receive(msg_buf, msg_len, msg_buf.Length - msg_len, SocketFlags.None);                
                msg_len += tmp_len;
            }
            msg_buf[msg_len] = 0;

            char[] msg_str = Encoding.ASCII.GetChars(msg_buf, 0, msg_len);
            string tempstr = new string(msg_str);
	        
            if (tempstr.StartsWith("$START")) 
            {
                // get out the probe_num
                string num_str = tempstr.Substring(7, msg_len - 8); 
                probe_num = int.Parse(num_str);
                if (verbose) {
                    Console.WriteLine("probe_num = {0} ", probe_num);
                }
                
                update_filter_list(newsd, probe_num, src_ip_str, dst_ip_str, out listening_port);
                
                tempstr = "$READY$$" + listening_port + "$";
                msg_buf = Encoding.ASCII.GetBytes(tempstr);
                msg_len = msg_buf.Length;
                newsd.Send(msg_buf);
                if (verbose) {
                    Console.WriteLine("listening port = {0} ", listening_port);
                }
            }
        }
    }
}
