/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Timers;
using System.Threading;

namespace MassTools.NetworkAwareness.AvailableBW
{
    public class AvailableBW_UDPing : MassTool
    {
        public event MassToolDataReadyEventHandler DataReadyEvent;

       	private Int32 m_nPacketSize = 64; // must larger than ulong
        private Int32 m_nPacketNum = 3;
        private Int32 m_nInterval = 500;

        private Socket mClientSock = null;
        private Socket mServerSock = null;
        private System.Timers.Timer sendingTimer = null;
        private System.Threading.Thread recvingThread = null;
        private System.Threading.Thread serverThread = null;
        private bool mbStarted = false;
        private IPEndPoint mRemoteEP = null;
        private const int UDPEchoPort = 8521; // the remote receiver side (server) port for sending probe
        private const int UDPEAckPort = 8522; // the local side (client) port for receiving ack
        private byte[] sendingbuff = null;

        #region server side

        public AvailableBW_UDPing()
        {
            // start the server side as each node is both server and client
            mServerSock = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            IPEndPoint ipPoint = new IPEndPoint(IPAddress.Any, UDPEchoPort);
            mServerSock.Bind(ipPoint);

            serverThread = new Thread(new ThreadStart(waitingfun));
            serverThread.Start();
        }

        private void waitingfun()
        {
            while (true)
            {
                // Creates an IpEndPoint to capture the identity of the sending host.
                IPEndPoint remoteIpPoint = new IPEndPoint(IPAddress.Any, 0);
                EndPoint tempRemoteEP = (EndPoint)remoteIpPoint;

                // Creates a byte buffer to receive the message.
                byte[] recvingbuff = new byte[m_nPacketSize]; //20(iph) + 22(icmph) + 32(data) = 74 < 128

                // Receives datagram from a remote host.  This call blocks!
                int nReceived = mServerSock.ReceiveFrom(recvingbuff, ref tempRemoteEP);

                mServerSock.SendTo(recvingbuff, tempRemoteEP);
            }
        }

        public void Dispose()
        {
            serverThread.Abort();
            serverThread = null;

            mServerSock.Close();
            mServerSock = null;
        }

        #endregion

        #region client side

        public override bool Start()
        {
            if (mbStarted) return true;
            sendingbuff = new byte[m_nPacketSize];

            // create socket
            mClientSock = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            IPEndPoint ipPoint = new IPEndPoint(IPAddress.Any, UDPEAckPort);
            mClientSock.Bind(ipPoint);

            // set receiving thread
            recvingThread = new Thread(new ThreadStart(recvfun));
            recvingThread.Start();

            // set sending timer
            sendingTimer = new System.Timers.Timer();
            sendingTimer.Elapsed += new ElapsedEventHandler(sendingTimer_Elapsed);
            sendingTimer.Interval = m_nInterval;
            sendingTimer.Start();

            mbStarted = true;
            return mbStarted;
        }

        public override bool Stop()
        {
            if (!mbStarted) return true;

            sendingTimer.Stop();
            sendingTimer = null;

            recvingThread.Abort();
            recvingThread = null;

            mClientSock.Close();
            mClientSock = null;

            mbStarted = false;
            return !mbStarted;
        }

        public void SetPacketSize(Int32 n)
        {
            m_nPacketSize = n;
        }

        public void SetPacketNum(Int32 n)
        {
            m_nPacketNum = n;
        }

        public void SetInterval(Int32 n)
        {
            m_nInterval = n;
        }

        public void SetRemoteAddr(string strAddr)
        {

            if (true == Char.IsDigit(strAddr[0]))
            {
                IPAddress ipAddr = IPAddress.Parse(strAddr);
                mRemoteEP = new IPEndPoint(ipAddr, UDPEchoPort);
            }
            else
            {
                IPHostEntry lipa = Dns.GetHostEntry(strAddr);
                mRemoteEP = new IPEndPoint(lipa.AddressList[0], UDPEchoPort);
            }
        }

        public void AddSubscriber(MassToolDataReadyEventHandler handler)
        {
            DataReadyEvent += handler;
        }

        private void sendingTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            //set up package with current time stamp
            byte[] timeStamp = BitConverter.GetBytes(DateTime.Now.Ticks);
            timeStamp.CopyTo(sendingbuff, 0);

            mClientSock.SendTo(sendingbuff, mRemoteEP);
        }

        private void recvfun()
        {
            while (true)
            {
                // check socket receive asnychronously and heed timeout
                if (true == mClientSock.Poll(1000 * m_nInterval * m_nPacketNum, SelectMode.SelectRead))
                {

                    // Creates an IpEndPoint to capture the identity of the sending host.
                    IPEndPoint remoteIpPoint = new IPEndPoint(IPAddress.Any, 0);
                    EndPoint tempRemoteEP = (EndPoint)remoteIpPoint;

                    // Creates a byte buffer to receive the message.
                    byte[] recvingbuff = new byte[m_nPacketSize]; //20(iph) + 22(icmph) + 32(data) = 74 < 128

                    // Receives datagram from a remote host.  This call blocks!
                    int nReceived = mClientSock.ReceiveFrom(recvingbuff, ref tempRemoteEP);

                    TimeSpan elapsedTime = new TimeSpan(DateTime.Now.Ticks - BitConverter.ToInt64(recvingbuff, 0));
                    int tempInterval = elapsedTime.Milliseconds;

                    if (tempInterval > m_nPacketNum * m_nInterval)
                    {
                        DataReadyEvent(this, new MassToolEventArgs(8));                        
                    }
                    else
                    {
                        DataReadyEvent(this, new MassToolEventArgs(12));                        
                    }                    
                }
                else
                {
                    DataReadyEvent(this, new MassToolEventArgs(8));
                }
            }
        }
        #endregion

    }
}
