/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Timers;
using System.Management;

namespace MassTools.DeviceAwareness.AvailableMemory
{
    public class AvailableMemory : MassTool
    {
        public event MassToolDataReadyEventHandler DataReadyEvent;

        System.Diagnostics.PerformanceCounter _perfCounter = null;

        private System.Timers.Timer testmemTimer = null;
        bool mbStarted = false;
        int m_nInterval = 500;

        public AvailableMemory()
        {
            _perfCounter = new System.Diagnostics.PerformanceCounter("Memory", "Available MBytes");
            // _perfCounter.CategoryName = "Memory";
            // _perfCounter.CounterName = "Available MBytes"; //Available Bytes

            testmemTimer = new System.Timers.Timer();
            testmemTimer.Elapsed += new ElapsedEventHandler(testmemTimer_Elapsed);

            mbStarted = false;
        }

        public override bool Start()
        {
            if (mbStarted) return true;

            testmemTimer.Interval = m_nInterval;
            testmemTimer.Start();

            mbStarted = true;
            return true;
        }

        public override bool Stop()
        {
            if (!mbStarted) return true;

            testmemTimer.Stop();

            mbStarted = false;
            return true;
        }

        public void SetInterval(Int32 n)
        {
            m_nInterval = n;
        }

        public void testmemTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            //"\Process()\% Processor Time" can go up to N*100 (where N is the number of CPUs) because it adds up the CPU usage of the requested process across all the CPUs.
            //"\Process(_Total)\% Processor Time" should always be around N*100 (where N is the number CPUs) because it adds up the CPU usage of each process, including the idle process.
            //"\Processor()\% Processor Time" can go up to 100 because its the CPU usage of the requested CPU.
            //"\Processor(_Total)\% Processor Time" can go up to 100 because its the average CPU usage across all CPUs.

            DataReadyEvent(this, new MassToolEventArgs(_perfCounter.NextValue()));
        }

        public void GetTotalMemorySize()
        {
            double totalCapacity = 0;
            ObjectQuery objectQuery = new ObjectQuery("select * from Win32_PhysicalMemory"); //Win32_Processor
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(objectQuery);
            ManagementObjectCollection vals = searcher.Get();

            foreach (ManagementObject val in vals)
            {
                totalCapacity += System.Convert.ToDouble(val.GetPropertyValue("Capacity"));
            }

            // Console.WriteLine("Total Machine Memory = " + totalCapacity.ToString() + " Bytes");
            // Console.WriteLine("Total Machine Memory = " + (totalCapacity / 1024) + " KiloBytes");
            // Console.WriteLine("Total Machine Memory = " + (totalCapacity / 1048576) + "    MegaBytes");
            // Console.WriteLine("Total Machine Memory = " + (totalCapacity / 1073741824) + " GigaBytes");
        }

    }
}
