/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace Masslets.ImageProcess
{
    public interface DataOutput
    {
        void AddSubscriber(MassletDataReadyEventHandler subs, bool bAdd);
    }

    interface DataInput
    {
        MassletDataReadyEventHandler GetSubscriber();
    }

    public class Display : Masslet, DataOutput, DataInput
    {
        IntPtr m_dispHandle;
        int m_width;
        int m_height;
        
        // Delegate instance for data ready and output data from PtrDataOutput interface
        public event MassletDataReadyEventHandler DataReadyEvent;

        // delegate instanc for data receiving
        public MassletDataReadyEventHandler DataRequestHandler;

        public Display()
        {
            
        }

        public Display(IntPtr p, int w, int h)
        {
            m_dispHandle = p;
            m_width = w;
            m_height = h;
        }
          

        void DataOutput.AddSubscriber(MassletDataReadyEventHandler subs, bool bAdd)
        {
            if (bAdd)
                DataReadyEvent += subs;
            else
                DataReadyEvent -= subs;
        }

        MassletDataReadyEventHandler DataInput.GetSubscriber()
        {
            if (DataRequestHandler == null)
                DataRequestHandler = new MassletDataReadyEventHandler(DataReceiveCallBack);
            return DataRequestHandler;
        }

        public void DataReceiveCallBack(object sender, MassletEventArgs e)
        {
            if (m_dispHandle != null)
            {
                Image im1 = Converters.byteArrayToImage(e.mData);
                Image im2 = im1.GetThumbnailImage(m_width, m_height, null, IntPtr.Zero);

                //PictureBox picBox = (PictureBox)Control.FromHandle(m_dispHandle);
                // reverse the image data, that's not very good for display normal data except webcam
                //picBox.Image = im2;

                Graphics g = Graphics.FromHwnd(m_dispHandle);
                g.DrawImage(im2, 0, 0, m_width, m_height);
            }
            if (DataReadyEvent != null)
            {
                DataReadyEvent(this, new MassletEventArgs(e.mData));
            }
        }

        public override bool Start()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override bool Stop()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void SetDisplayInf(IntPtr p, int w, int h)
        {
            m_dispHandle = p;
            m_width = w;
            m_height = h;
        }
    }
}
