/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

/// <summary>
/// Summary description for Class1
/// </summary>
/// 
namespace Masslets.ImageProcess
{
    public class Converters
    {
        // Public methods
        #region structure and byte array convertion

        public static object GetStructure(IntPtr ptr, ValueType structure)
        {
            return Marshal.PtrToStructure(ptr, structure.GetType());
        }

        public static object GetStructure(int ptr, ValueType structure)
        {
            return GetStructure(new IntPtr(ptr), structure);
        }

        public static void Copy(IntPtr ptr, byte[] data)
        {
            Marshal.Copy(ptr, data, 0, data.Length);
        }

        public static void Copy(int ptr, byte[] data)
        {
            Copy(new IntPtr(ptr), data);
        }

        public static int SizeOf(object structure)
        {
            return Marshal.SizeOf(structure);
        }

        public static byte[] StructureToByteArray(ValueType obj)
        {
            int len = Marshal.SizeOf(obj);
            byte[] arr = new byte[len];

            IntPtr ptr = Marshal.AllocHGlobal(len);
            Marshal.StructureToPtr(obj, ptr, true);
            Marshal.Copy(ptr, arr, 0, len);
            Marshal.FreeHGlobal(ptr);

            return arr;
        }

        public static object ByteArrayToStructure(byte[] bytearray, ValueType obj)
        {
            int len = Marshal.SizeOf(obj);

            IntPtr i = Marshal.AllocHGlobal(len);
            Marshal.Copy(bytearray, 0, i, len);
            object ob = Marshal.PtrToStructure(i, obj.GetType());
            Marshal.FreeHGlobal(i);
            return ob;
        }

        #endregion



        #region image and byte array conversion

        public static byte[] imageToByteArray(Image imageIn, ImageFormat format)
        {
            MemoryStream ms = new MemoryStream();
            imageIn.Save(ms, format);
            return ms.ToArray();
        }

        public static Image byteArrayToImage(byte[] byteArrayIn)
        {
            MemoryStream ms = new MemoryStream(byteArrayIn);
            Image returnImage = Image.FromStream(ms);
            return returnImage;
        }

        public static Image dataToImage(int width, int height, int lpdata)
        {
            Image im = new Bitmap(width, height, width * 3, PixelFormat.Format24bppRgb, (IntPtr)lpdata);
            //im.RotateFlip(RotateFlipType.Rotate180FlipX);
            return im;
        }

        public static byte[] imageToJpegByteArray(Image imageIn, long qulity)
        {
            ImageCodecInfo ici = null;
            ImageCodecInfo[] codecs = ImageCodecInfo.GetImageEncoders();
            foreach (ImageCodecInfo codec in codecs)
            {
                if (codec.MimeType == "image/jpeg") ici = codec;
            }
            EncoderParameters ep = new EncoderParameters(1);
            ep.Param[0] = new EncoderParameter(Encoder.Quality, qulity);

            MemoryStream ms = new MemoryStream();
            imageIn.Save(ms, ici, ep);
            return ms.ToArray();
        }

        public static Image StretchImage(Image imageIn, int width, int height, bool bratio)
        {
            if (imageIn.Width < width && imageIn.Height < height) return imageIn;

            Image imageOut = null;
            if (bratio)
            {
                double rw = (double)(imageIn.Width) / (double)width;
                double rh = (double)(imageIn.Height) / (double)height;

                if (rw > rh)
                    imageOut = imageIn.GetThumbnailImage(width, (int)(imageIn.Height / rw), null, IntPtr.Zero);
                else
                    imageOut = imageIn.GetThumbnailImage((int)(imageIn.Width / rh), height, null, IntPtr.Zero);
            }
            else
            {
                if (imageIn.Width > width && imageIn.Height > height)
                {
                    imageOut = imageIn.GetThumbnailImage(width, height, null, System.IntPtr.Zero);
                }
                else if (imageIn.Width > 640)
                {
                    imageOut = imageIn.GetThumbnailImage(width, imageIn.Height, null, IntPtr.Zero);
                }
                else if (imageIn.Height > 480)
                {
                    imageOut = imageIn.GetThumbnailImage(imageIn.Width, height, null, IntPtr.Zero);
                }
            }
            return imageOut;
        }

        #endregion

    }
}
