/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace Masslets.ImageProcess
{
    // 
    public interface PtrDataOutput
    {
        void AddSubscriber(ImageRefReadyEventHandler subs, bool bAdd);
    }

    public interface ImageDataOutput
    {
        void AddSubscriber(MassletDataReadyEventHandler subs, bool bAdd);
    }

    //Web Camera Class
    public class WebCam : Masslet, PtrDataOutput, ImageDataOutput
    {
#region variables
        public enum GrabType { tPreview = 0, tFrame };
        // constant value, support: 800*600, 640*480, 352*288, 320*240, 176*144, 160*120
        static int[] SUPPORTEDWIDTH = { 800, 640, 352, 320, 176, 160 };
        static int[] SUPPORTEDHEIGHT = { 600, 480, 288, 240, 144, 120 };

        // parameters dynamically set by the configuration file        
        private int VIDEOFRAMEWIDTH = 320;
        private int VIDEOFRAMEHEIGHT = 240;
        private short CAPFREQENCY = 66;

        // parameters set by the application
        private IntPtr mControlPtr; // Holds the managed pointer of the control
        private int mWidth;
        private int mHeight;

        // internal parameters
        private IntPtr lwndC; // Holds the unmanaged handle of the control
        API32.BITMAPINFO mBitmapinfo;
        private bool mbStarted;

        // Delegate instance for the frame callback - must keep alive! gc should NOT collect it
        private API32.FrameEventHandler mFrameEventHandler;

        // Delegate instance for data ready and output data from PtrDataOutput interface
        public event ImageRefReadyEventHandler PtrDataReadyEvent;
        public event MassletDataReadyEventHandler ImageDataReadyEvent;

#endregion 

        
#region Constructurs
        public WebCam()
        {
            mbStarted = false;
        }

        public WebCam(IntPtr handle, int width, int height)
        {
            mControlPtr = handle;
            mWidth = width;
            mHeight = height;
            mbStarted = false;
        }
#endregion

#region self defined functions
        // start the web camera
        public bool StartWebCam(GrabType type)
        {
            if (mbStarted) return true;

            byte[] lpszName = new byte[100];
            byte[] lpszVer = new byte[100];

            if (!API32.capGetDriverDescriptionA(0, lpszName, 100, lpszVer, 100)) return false;

            if (type == GrabType.tFrame)
            {  // this one is not efficent, just for click event response.
                this.lwndC = API32.capCreateCaptureWindowA(lpszName, 0, 0, 0, mWidth, mHeight, IntPtr.Zero, 0);
            }
            else if (type == GrabType.tPreview)
            {  // use this one if possible
                this.lwndC = API32.capCreateCaptureWindowA(lpszName, API32.WS_VISIBLE + API32.WS_CHILD, 0, 0, mWidth, mHeight, mControlPtr, 0);
            }

            if (this.lwndC == null) return false;

            if (this.capDriverConnect(this.lwndC, 0))
            {
                mBitmapinfo = new API32.BITMAPINFO();
                this.capGetVideoFormat(this.lwndC, ref mBitmapinfo, Converters.SizeOf(mBitmapinfo));

                if (Array.IndexOf(SUPPORTEDWIDTH, VIDEOFRAMEWIDTH) == Array.IndexOf(SUPPORTEDHEIGHT, VIDEOFRAMEHEIGHT) &&
                    mBitmapinfo.bmiHeader.biWidth != VIDEOFRAMEWIDTH )
                {
                    mBitmapinfo.bmiHeader.biWidth = VIDEOFRAMEWIDTH;
                    mBitmapinfo.bmiHeader.biHeight = VIDEOFRAMEHEIGHT;
                    this.capSetVideoFormat(this.lwndC, ref mBitmapinfo, Converters.SizeOf(mBitmapinfo));
                }

                this.mFrameEventHandler = new API32.FrameEventHandler(FrameCallBack);
                this.capSetCallbackOnFrame(this.lwndC, this.mFrameEventHandler);
                API32.SetWindowPos(this.lwndC, 0, 0, 0, mWidth, mHeight, 6);

                if (type == GrabType.tFrame)
                {
                    capGrabFrame(this.lwndC);
                }
                else if (type == GrabType.tPreview)
                {
                    this.capPreviewRate(this.lwndC, CAPFREQENCY);
                    this.capPreviewScale(this.lwndC, true);
                    this.capPreview(this.lwndC, true);
                }
                
                mbStarted = true;
            }
            return true;
        }

        // Close the web camera
        public void CloseWebcam()
        {
            if (mbStarted)
            {
                this.capDriverDisconnect(this.lwndC);
                mbStarted = false;
            }
        }

        public void GetOneFrame()
        {
            if (mbStarted)
                capGrabFrame(this.lwndC);
        }

        // private functions
        private bool capDriverConnect(IntPtr lwnd, short i)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_DRIVER_CONNECT, i, 0);
        }

        private bool capDriverDisconnect(IntPtr lwnd)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_DRIVER_DISCONNECT, 0, 0);
        }

        private bool capPreview(IntPtr lwnd, bool f)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_SET_PREVIEW, f, 0);
        }

        private bool capPreviewRate(IntPtr lwnd, short wMS)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_SET_PREVIEWRATE, wMS, 0);
        }

        private bool capPreviewScale(IntPtr lwnd, bool f)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_SET_SCALE, f, 0);
        }

        private bool capGrabFrame(IntPtr lwnd)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_GRAB_FRAME, 0, 0);
        }

        private bool capSetCallbackOnFrame(IntPtr lwnd, API32.FrameEventHandler lpProc)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_SET_CALLBACK_FRAME, 0, lpProc);
        }

        private bool capGetVideoFormat(IntPtr lwnd, ref API32.BITMAPINFO BmpFormat, int CapFormatSize)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_GET_VIDEOFORMAT, CapFormatSize, ref BmpFormat);
        }

        private bool capSetVideoFormat(IntPtr lwnd, ref API32.BITMAPINFO BmpFormat, int CapFormatSize)
        {
            return API32.SendMessage(lwnd, API32.WM_CAP_SET_VIDEOFORMAT, CapFormatSize, ref BmpFormat);
        }

        private void FrameCallBack(IntPtr lwnd, IntPtr lpVHdr)
        {
            API32.VIDEOHDR videoHeader = new API32.VIDEOHDR();
            videoHeader = (API32.VIDEOHDR)Converters.GetStructure(lpVHdr, videoHeader);

            Image im = Converters.dataToImage(mBitmapinfo.bmiHeader.biWidth, mBitmapinfo.bmiHeader.biHeight, videoHeader.lpData);
            im.RotateFlip(RotateFlipType.Rotate180FlipX);

            // if local window is set, show the image to local window
            // do not need to show, already preview it
            /*if (mControlPtr != null)
            {
                Graphics g = Graphics.FromHwnd(mControlPtr);
                g.DrawImage(im, 0, 0, mWidth, mHeight);
            }
            */
            if (PtrDataReadyEvent != null)
            {
                //ProcessPtrData(lpVHdr);                
                PtrDataReadyEvent(this, new ImageEventArgs(im));
            }

            // if there are other output, add more processing function here
            if (ImageDataReadyEvent != null)
            {
                //ProcessImageData(lpVHdr);
                byte[] data = Converters.imageToByteArray(im, ImageFormat.Bmp);
                ImageDataReadyEvent(this, new MassletEventArgs(data));
            }
        }

        private void ProcessPtrData(IntPtr lpVHdr)
        {
            API32.VIDEOHDR videoHeader = new API32.VIDEOHDR();

            videoHeader = (API32.VIDEOHDR)Converters.GetStructure(lpVHdr, videoHeader);

            IMAGEMSG msg = new IMAGEMSG(mBitmapinfo.bmiHeader.biWidth, 
                mBitmapinfo.bmiHeader.biHeight, videoHeader.lpData, videoHeader.dwBytesUsed);

            MassletEventArgs e = new MassletEventArgs(Converters.StructureToByteArray(msg));

            //PtrDataReadyEvent(this, e);
        }

        private void ProcessImageData(IntPtr lpVHdr)
        {
            API32.VIDEOHDR videoHeader = new API32.VIDEOHDR();

            videoHeader = (API32.VIDEOHDR)Converters.GetStructure(lpVHdr, videoHeader);

            Image im = Converters.dataToImage(mBitmapinfo.bmiHeader.biWidth, mBitmapinfo.bmiHeader.biHeight, videoHeader.lpData);
            byte[] data = Converters.imageToByteArray(im, ImageFormat.Bmp);

            //ImageDataReadyEvent(this, new MassletEventArgs(data));
        }
#endregion

        
#region override functions
        public override bool Start()
        {
            StartWebCam(GrabType.tPreview);
            return true;
        }

        public override bool Stop()
        {
            CloseWebcam();
            return true;
        }

        void PtrDataOutput.AddSubscriber(ImageRefReadyEventHandler subs, bool bAdd)
        {
            if (bAdd) 
                PtrDataReadyEvent += subs;
            else
                PtrDataReadyEvent -= subs;
        }

        void ImageDataOutput.AddSubscriber(MassletDataReadyEventHandler subs, bool bAdd)
        {
            if (bAdd)
                ImageDataReadyEvent += subs;
            else
                ImageDataReadyEvent -= subs;
        }


#endregion 

        
#region parameter setting functions
        public void SetCaptureWidth(int w)
        {
            if (w == VIDEOFRAMEWIDTH) return;
            int iw = Array.IndexOf(SUPPORTEDWIDTH, w);
            if (iw == -1) return;
            VIDEOFRAMEWIDTH = w;
            if (iw == Array.IndexOf(SUPPORTEDHEIGHT, VIDEOFRAMEHEIGHT) && mbStarted)
            {
                CloseWebcam();
                StartWebCam(GrabType.tPreview);
            }
        }
        public void SetCaptureHeight(int h)
        {
            if (h == VIDEOFRAMEHEIGHT) return;
            int ih = Array.IndexOf(SUPPORTEDHEIGHT, h);
            if (ih == -1) return;
            VIDEOFRAMEHEIGHT = h;
            if (ih == Array.IndexOf(SUPPORTEDWIDTH, VIDEOFRAMEWIDTH) && mbStarted)
            {
                CloseWebcam();
                StartWebCam(GrabType.tPreview);
            }
        }
        public void SetCaptureFreq(int f)
        {
            if (CAPFREQENCY == f) return;
            CAPFREQENCY = (short)f;
            if (mbStarted)
            {
                CloseWebcam();
                StartWebCam(GrabType.tPreview);
            }
        }

        public void SetLocalWindowInf(IntPtr handle, int w, int h)
        {
            mControlPtr = handle;
            mWidth = w;
            mHeight = h;
        }

#endregion
    }

}