/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Timers;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace Masslets.ImageProcess
{
    public interface PtrDataOutput
    {
        void AddSubscriber(ImageRefReadyEventHandler subs, bool bAdd);
    }

    public interface ImageDataOutput
    {
        void AddSubscriber(MassletDataReadyEventHandler subs, bool bAdd);
    }

    public class WindowCap : Masslet, PtrDataOutput, ImageDataOutput
    {
        // Delegate instance for data ready and output data from PtrDataOutput interface
        public event ImageRefReadyEventHandler PtrDataReadyEvent;
        public event MassletDataReadyEventHandler ImageDataReadyEvent;

        private int m_nSampleFreq = 500; //ms
        private System.Timers.Timer m_tSampleTimer = null;
        private bool m_bStarted = false;
        private FormWindow m_fCapWind = null;

        //private SplitterPanel m_pDispControl = null;
        private IntPtr m_pDispHandle = IntPtr.Zero;
        private int m_nDispWidth, m_nDispHeight;

        public WindowCap()
        {
            m_tSampleTimer = new System.Timers.Timer();
            m_tSampleTimer.Elapsed += new ElapsedEventHandler(sampleTimer_Elapsed);
            m_fCapWind = new FormWindow();
        }

        public void Dispose()
        {
            m_tSampleTimer.Close();
            m_fCapWind.Close();
            GC.Collect();
        }

        #region override functions
        public override bool Start()
        {
            if (m_bStarted) return true;
            m_fCapWind.Show();
            m_tSampleTimer.Interval = m_nSampleFreq;
            m_tSampleTimer.Start();
            m_bStarted = true;
            return true;
        }

        public override bool Stop()
        {
            if (!m_bStarted) return true;
            m_tSampleTimer.Stop();
            m_fCapWind.Hide();
            m_bStarted = false;
            return true;
        }

        void PtrDataOutput.AddSubscriber(ImageRefReadyEventHandler subs, bool bAdd)
        {
            if (bAdd)
                PtrDataReadyEvent += subs;
            else
                PtrDataReadyEvent -= subs;
        }

        void ImageDataOutput.AddSubscriber(MassletDataReadyEventHandler subs, bool bAdd)
        {
            if (bAdd)
                ImageDataReadyEvent += subs;
            else
                ImageDataReadyEvent -= subs;
        }
        #endregion 

        #region self defined functions
        public void SetCaptureFreq(int f)
        {
            m_nSampleFreq = f;
            if (m_bStarted)
            {
                m_tSampleTimer.Stop();
                m_tSampleTimer.Interval = m_nSampleFreq;
                m_tSampleTimer.Start();
            }
        }

        public void SetCaptureWidth(int w)
        {
        }
        public void SetCaptureHeight(int h)
        {
        }

        public void SetLocalWindowInf(IntPtr handle, int w, int h)
        {
            m_pDispHandle = handle;
            //m_pDispControl = (SplitterPanel)Control.FromHandle(handle);
            m_nDispWidth = w;
            m_nDispHeight = h;
        }

        private void sampleTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (m_fCapWind.Visible)
            {
                Bitmap im = WindowCapture.CaptureWindow.GetClientImage(m_fCapWind.Handle);

                // if local window is set, show the image to local window
                if (m_pDispHandle != null)
                {
                    //Graphics g = m_pDispControl.CreateGraphics();
                    Graphics g = Graphics.FromHwnd(m_pDispHandle);
                    g.DrawImage(im, 0, 0, m_nDispWidth, m_nDispHeight);
                }
                
                if (PtrDataReadyEvent != null)
                {
                    PtrDataReadyEvent(this, new ImageEventArgs(im));
                }

                // if there are other output, add more processing function here
                if (ImageDataReadyEvent != null)
                {
                    byte[] data = Converters.imageToByteArray(im, ImageFormat.Bmp);
                    ImageDataReadyEvent(this, new MassletEventArgs(data));
                }
            }
        }
        #endregion
    }
}
