/**
 * Copyright (c) 2009 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/

// modsDlg.cpp : implementation file
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using MassTools;
using System.Threading;

namespace VideoConferencing
{
    public partial class FormVideo : Form
    {
        public delegate void UpdateTextCallback(string text);
        private MassWare.MassWare massAgent = null;
        private string m_strIP = "";

        private double m_vbw = 0, m_vcpu = 0;

        public FormVideo()
        {
            InitializeComponent();
        }

        private void InitializeMassWare()
        {
            massAgent = new MassWare.MassWare(@"D:\MassWare\Outputs\vcRules.xml");
            massAgent.Initialize();
            massAgent.AddArchSubscriber(new MassWare.Architecture.MassWareArchChangeEventHandler(MassWareArchChanged));

            object[] paramlist = { splitContainer1.Panel1.Handle, splitContainer1.Panel1.Width, splitContainer1.Panel1.Height };
            massAgent.SetAppParameter("Masslets.ImageProcess.WindowCap", "SetLocalWindowInf", paramlist);
            massAgent.SetAppParameter("Masslets.ImageProcess.WebCam", "SetLocalWindowInf", paramlist);
            paramlist = new object[3] { splitContainer1.Panel2.Handle, splitContainer1.Panel2.Width, splitContainer1.Panel2.Height };
            massAgent.SetAppParameter("Masslets.ImageProcess.Display", "SetDisplayInf", paramlist);

            massAgent.SetAppParameter("MassTools.NetworkAwareness.AvailableBW.AvailableBW", "AddSubscriber", new object[] { new MassToolDataReadyEventHandler(BWDataReceiveCallBack) });
            massAgent.SetAppParameter("MassTools.DeviceAwareness.AvailableCPU.AvailableCPU", "AddSubscriber", new object[] { new MassToolDataReadyEventHandler(CPUDataReceiveCallBack) });

            if (m_strIP != "")
            {
                massAgent.SyncTo(IPAddress.Parse(m_strIP));
                massAgent.SetAppParameter("MassTools.NetworkAwareness.AvailableBW.AvailableBW", "SetRemoteAddr", new object[] { m_strIP });
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            InitializeMassWare();
        }

        private void connectToToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            FormConnect formConn = new FormConnect();
            if (formConn.ShowDialog(this) == DialogResult.OK)
            {
                m_strIP = formConn.mStrIP;
                massAgent.SyncTo(IPAddress.Parse(m_strIP));
                massAgent.SetAppParameter("MassTools.NetworkAwareness.AvailableBW.AvailableBW", "SetRemoteAddr", new object[] { m_strIP });
            }
            formConn.Dispose();
        }

        private void startToolStripMenuItem_Click(object sender, EventArgs e)
        {
            massAgent.Start();
        }

        private void stopToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (massAgent != null)
                massAgent.Stop();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            massAgent.Dispose();
            GC.Collect();
        }

        private void MassWareArchChanged(object sender, MassWare.MassWareArchEventArgs e)
        {
            string[] dispStr = e.mMessage.Split('$');
            string txt = "";
            if (e.mType == MassWare.MassWareArchEventArgs.PROACTIVE_CHANGE)
                txt = "Proactive Actuator Changed: " + dispStr[0] + " -> Sender" + dispStr[1];
            else if (e.mType == MassWare.MassWareArchEventArgs.REACTIVE_CHANGE)
                txt = "Reactive Actuator Changed: " + "Receiver -> " + dispStr[0];

            textBoxOutput.Invoke(new UpdateTextCallback(AppendText), new object[] { txt+"\r\n" });
        }

        public void BWDataReceiveCallBack(object sender, MassToolEventArgs e)
        {
            m_vbw = e.mValue;
            string strDisp;
            if (0 == m_vcpu)
                strDisp = String.Format("Measured Bandwidth: {0}Mbps\r\n", m_vbw);
            else
                strDisp = String.Format("Measured Bandwidth: {0}Mbps\t CPU: {1}GHz\r\n", m_vbw, m_vcpu);

            textBoxOutput.Invoke(new UpdateTextCallback(AppendText), new object[] { strDisp });
        }
        public void CPUDataReceiveCallBack(object sender, MassToolEventArgs e)
        {
            m_vcpu = e.mValue;
            string strDisp;
            if (0 == m_vbw)
                strDisp = String.Format("Measured CPU: {0}GHz\r\n", m_vcpu);
            else
                strDisp = String.Format("Measured Bandwidth: {0}Mbps\t CPU: {1}GHz\r\n",m_vbw, m_vcpu);
            textBoxOutput.Invoke(new UpdateTextCallback(AppendText), new object[] { strDisp });
        }
        private void AppendText(string text)
        {
            // Set the textbox text.
            textBoxOutput.AppendText(text);
        }

        private void restartToolStripMenuItem_Click(object sender, EventArgs e)
        {
            massAgent.Dispose();
            massAgent = null;
            textBoxOutput.Clear();
            GC.Collect();
            InitializeMassWare();
            massAgent.Start();
        }

    }
}